@@ -11,5 +11,5 @@ using namespace Rcpp;
 
 double egonet_attrs_cpp(
     const arma::sp_mat & graph, const arma::uvec V,
-    NumericMatrix attrs, bool outer=true, bool self=true, bool valued=true);
+    bool outer=true, bool self=true, bool valued=true);
 #endif
